/*
  IM50175H 14-segment LCD driver
  Modified for voltmeter
*/


#include <avr/io.h>


#define LATCH_DDR DDRD
#define LATCH_PORT PORTD
#define LATCH_BIT PD5
#define DATA_DDR DDRD
#define DATA_PORT PORTD
#define DATA_BIT PD6
#define CLK_DDR DDRD
#define CLK_PORT PORTD
#define CLK_BIT PD7


.text


send:
	ldi	r25, 4
1:	sbrc	r24, 0
	sbi	_SFR_IO_ADDR(DATA_PORT), DATA_BIT
	lsr	r24
	sbi	_SFR_IO_ADDR(CLK_PORT), CLK_BIT
	dec	r25
	cbi	_SFR_IO_ADDR(CLK_PORT), CLK_BIT
	cbi	_SFR_IO_ADDR(DATA_PORT), DATA_BIT
	brne	1b
	ret


.global	lcd_init
.func lcd_init
lcd_init:
	cbi	_SFR_IO_ADDR(LATCH_PORT), LATCH_BIT
	cbi	_SFR_IO_ADDR(DATA_PORT), DATA_BIT
	cbi	_SFR_IO_ADDR(CLK_PORT), CLK_BIT
	sbi	_SFR_IO_ADDR(LATCH_DDR), LATCH_BIT
	sbi	_SFR_IO_ADDR(DATA_DDR), DATA_BIT
	sbi	_SFR_IO_ADDR(CLK_DDR), CLK_BIT
	ret
.endfunc


.global	lcd_refresh
.func lcd_refresh
lcd_refresh:
	; 159-128
	ldi	XL,lo8(lcd_buf)
	ldi	XH,hi8(lcd_buf)
1:	ld	ZL, X+
	ldi	ZH, 0x00
	subi	ZL, lo8(-(tbl0))
	sbci	ZH, hi8(-(tbl0))
#ifdef __AVR_HAVE_LPMX__
	lpm	r24, Z
#else
	lpm
	mov	r24, r0
#endif
	rcall	send
	cpi	XL,lo8(lcd_buf + 8)
	brne	1b
	; 127-120
	ld	r24, X
	lsl	r24
	rcall	send
	rcall	send
	; 119-88
2:	ld	ZL, -X
	ldi	ZH, 0x00
	subi	ZL, lo8(-(tbl0))
	sbci	ZH, hi8(-(tbl0))
#ifdef __AVR_HAVE_LPMX__
	lpm	r24, Z
#else
	lpm
	mov	r24, r0
#endif
	swap	r24
	rcall	send
	cpi	XL,lo8(lcd_buf)
	brne	2b
	; 87-80
	ldi	r24, 0x00
	rcall	send
	rcall	send
	; 79-48
3:	ld	ZL, X+
	ldi	ZH, 0x00
	subi	ZL, lo8(-(tbl1))
	sbci	ZH, hi8(-(tbl1))
#ifdef __AVR_HAVE_LPMX__
	lpm	r24, Z
#else
	lpm
	mov	r24, r0
#endif
	rcall	send
	cpi	XL, lo8(lcd_buf + 8)
	brne	3b
	; 47-40
	ld	r24, X
	rcall	send
	rcall	send
	; 39-8
4:	ld	ZL, -X
	ldi	ZH, 0x00
	subi	ZL, lo8(-(tbl1))
	sbci	ZH, hi8(-(tbl1))
#ifdef __AVR_HAVE_LPMX__
	lpm	r24, Z
#else
	lpm
	mov	r24, r0
#endif
	swap	r24
	rcall	send
	cpi	XL,lo8(lcd_buf)
	brne	4b
	; 7-0
	rcall	send
	rcall	send

	sbi	_SFR_IO_ADDR(LATCH_PORT), LATCH_BIT
	cbi	_SFR_IO_ADDR(LATCH_PORT), LATCH_BIT

	ret
.endfunc


.section	.bss
.global	lcd_buf
lcd_buf:
	.byte	0
	.byte	0
	.byte	0
	.byte	0
	.byte	0
	.byte	0
	.byte	0
	.byte	0
	.byte	0


.section	.text
tbl0:
	.byte	0b00000000	; 0
	.byte	0b00000000	; 1
	.byte	0b00001001	; 2
	.byte	0b00001001	; 3
	.byte	0b00001001	; 4
	.byte	0b00001001	; 5
	.byte	0b00001001	; 6
	.byte	0b00000000	; 7
	.byte	0b00001001	; 8
	.byte	0b00001001	; 9
	.byte	0b00000000	; space
	.byte	0b00000000	; period
	.byte	0b00001001	; minus
	.byte	0b01001000	; B
	.byte	0b00001001	; F
	.byte	0b10100000	; M
	.byte	0b00001101	; R
	.byte	0b00001001	; S
	.byte	0b00100010	; V
	.byte	0b00000110	; W
	.byte	0b00001001	; d
	.byte	0b00001001	; m
	.byte	0b00001000	; n
	.byte	0b01001001	; u
tbl1:
	.byte	0b11101101	; 0
	.byte	0b00101000	; 1
	.byte	0b01100101	; 2
	.byte	0b01101100	; 3
	.byte	0b10101000	; 4
	.byte	0b11001100	; 5
	.byte	0b11001101	; 6
	.byte	0b11101000	; 7
	.byte	0b11101101	; 8
	.byte	0b11101100	; 9
	.byte	0b00000000	; space
	.byte	0b00000010	; period
	.byte	0b00000000	; minus
	.byte	0b01101110	; B
	.byte	0b11000001	; F
	.byte	0b10101001	; M
	.byte	0b11100001	; R
	.byte	0b11001100	; S
	.byte	0b10000001	; V
	.byte	0b10101001	; W
	.byte	0b00101101	; d
	.byte	0b00001011	; m
	.byte	0b00001010	; n
	.byte	0b10100001	; u
